//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Fit/FitParameterHelper.cpp
//! @brief     Implements namespace FitModelHelper
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Fit/FitParameterHelper.h"
#include "GUI/Model/Item/FitParameterContainerItem.h"
#include "GUI/Model/Item/FitParameterItem.h"
#include "GUI/Model/Item/FitParameterLinkItem.h"
#include "GUI/Model/Item/JobItem.h"
#include "GUI/Model/Item/ParameterTreeItems.h"

ParameterItem* FitParameterHelper::getParameterItem(FitParameterContainerItem* container,
                                                    const QString& link)
{
    // Link is relative to ParameterContainerItem, so first we have to find it
    SessionItem* cur = container;
    while (cur && cur->modelType() != JobItem::M_TYPE)
        cur = cur->parent();
    ASSERT(cur && cur->modelType() == JobItem::M_TYPE);
    auto* jobItem = dynamic_cast<JobItem*>(cur);
    ASSERT(jobItem);
    return jobItem->parameterContainerItem()->findParameterItem(link);
}
