//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Item/LayerRoughnessItems.cpp
//! @brief     Implements classes LayerRoughnessItems
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Item/LayerRoughnessItems.h"

#include "GUI/Model/Types/DoubleDescriptor.h"
#include "GUI/Model/XML/Serializer.h"
#include "GUI/Model/Model/SessionXML.h"

using namespace GUI::Session::XML;

namespace {

namespace Tags {
const QString Uid("Uid");
} // namespace Tags
} // namespace

LayerBasicRoughnessItem::LayerBasicRoughnessItem()
{
    m_sigma.init("Sigma", "rms of the roughness", 1.0, Unit::nanometer, "sigma");
    m_hurst.init("Hurst",
                 "Hurst parameter which describes how jagged the interface,\n "
                 "dimensionless [0.0, 1.0], where 0.0 gives more spikes, \n1.0 more smoothness.",
                 0.3, Unit::unitless, 3, RealLimits::limited(0.0, 1.0), "hurst");
    m_lateralCorrelationLength.init("Correlation length",
                                    "Lateral correlation length of the roughness", 5.0,
                                    Unit::nanometer, "corrLen");
}

DoubleDescriptor LayerBasicRoughnessItem::sigma() const
{
    return m_sigma;
}

DoubleDescriptor LayerBasicRoughnessItem::hurst() const
{
    return m_hurst;
}

DoubleDescriptor LayerBasicRoughnessItem::lateralCorrelationLength() const
{
    return m_lateralCorrelationLength;
}

void LayerBasicRoughnessItem::serialize(Serializer& s)
{
    s.assertVersion(0);
    s.rw(m_sigma);
    s.rw(m_hurst);
    s.rw(m_lateralCorrelationLength);
}
