//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Model/SessionXML.h
//! @brief     Defines reader and writer classes for SessionModel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_MODEL_SESSIONXML_H
#define BORNAGAIN_GUI_MODEL_MODEL_SESSIONXML_H

#include "GUI/Model/Group/SelectionProperty.h"
#include "GUI/Model/Types/DoubleDescriptor.h"
#include <QVariant>
#include <QXmlStreamWriter> // used in every including file
#include <heinz/Complex.h>
#include <heinz/Vectors3D.h>

class QXmlStreamReader;
class SessionItem;
class MessageService;
class QUuid;

namespace GUI::Session::XML {

constexpr auto ItemMimeType = "application/org.bornagainproject.xml.item.z";
constexpr auto LinkMimeType = "application/org.bornagainproject.fittinglink";
constexpr auto InstrumentModelTag("InstrumentModel");
constexpr auto JobModelTag("JobModel");
constexpr auto RealDataModelTag("RealDataModel");
constexpr auto ItemTag("Item");
constexpr auto ModelTypeAttribute("ModelType");
constexpr auto ParameterTag("Parameter");

void gotoEndElementOfTag(QXmlStreamReader* reader, const QString& tag);

void writeModel(QXmlStreamWriter* writer, SessionItem* modelRootItem);
void writeItemAndChildItems(QXmlStreamWriter* writer, const SessionItem* item);

void readItems(QXmlStreamReader* reader, SessionItem* parent, QString topTag = "",
               MessageService* messageService = nullptr);
QString readProperty(QXmlStreamReader* reader, SessionItem* item);

} // namespace GUI::Session::XML

#endif // BORNAGAIN_GUI_MODEL_MODEL_SESSIONXML_H
