//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Plot/DataPropertyContainer.cpp
//! @brief     Implements class DataPropertyContainer
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Plot/DataPropertyContainer.h"
#include "GUI/Model/Plot/DataProperties.h"
#include "GUI/Model/Item/DataItem.h"
#include "GUI/Util/Error.h"

DataPropertyContainer::DataPropertyContainer()
    : SessionItem(M_TYPE)
{
    registerTag(T_CHILDREN, 0, -1, {Data1DProperties::M_TYPE});
    setDefaultTag(T_CHILDREN);
}

QVector<Data1DProperties*> DataPropertyContainer::propertyItems()
{
    QVector<Data1DProperties*> result;
    auto source = getItems();
    std::transform(source.begin(), source.end(), std::back_inserter(result),
                   [](SessionItem* item) { return dynamic_cast<Data1DProperties*>(item); });
    return result;
}

Data1DProperties* DataPropertyContainer::propertyItem(size_t i) const
{
    auto children = getItems();
    if (children.empty())
        return nullptr;
    auto* property_item = dynamic_cast<Data1DProperties*>(children[static_cast<int>(i)]);
    ASSERT(property_item);
    return property_item;
}

DataItem* DataPropertyContainer::basicDataItem() const
{
    auto* basic_property_item = propertyItem(0);
    if (!basic_property_item)
        return nullptr;
    return basic_property_item->dataItem();
}

void DataPropertyContainer::addItem(DataItem* data_item)
{
    if (this->model() != data_item->model())
        throw Error("Error in DataPropertyContainer::addItem: hosting models are different");

    auto property_items = propertyItems();
    Data1DProperties* previous_item = nullptr;
    if (!propertyItems().empty())
        previous_item = dynamic_cast<Data1DProperties*>(propertyItems().back());

    auto* property_item = new Data1DProperties();
    insertChild(-1, property_item);
    property_item->setDataItem(data_item);
    property_item->setColorProperty(Data1DProperties::nextColorName(previous_item));
}

std::vector<DataItem*> DataPropertyContainer::dataItems()
{
    std::vector<DataItem*> result;
    auto items = propertyItems();
    std::transform(items.begin(), items.end(), std::back_inserter(result),
                   [](Data1DProperties* item) {
                       auto* data_item = item->dataItem();
                       ASSERT(data_item);
                       return data_item;
                   });
    return result;
}

DataItem* DataPropertyContainer::dataItem(size_t i) const
{
    return propertyItem(i)->dataItem();
}
