//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Plot/DataViewUtils.cpp
//! @brief     Implements functions in namespace DataViewUtils
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Plot/DataViewUtils.h"
#include "Device/Coord/ICoordSystem.h"
#include "GUI/Model/Item/JobItem.h"
#include "GUI/Model/Item/InstrumentItems.h"
#include "GUI/Model/Item/Data1DViewItem.h"
#include "GUI/Model/Item/DataItem.h"
#include "GUI/Util/CoordName.h"

namespace {

ICoordSystem* getConverter(Data1DViewItem* view_item)
{
    auto* job_item = view_item->jobItem();
    ASSERT(job_item->instrumentItem());

    return job_item->instrumentItem()->createCoordSystem();
}

Axes::Coords selectedUnits(Data1DViewItem* view_item)
{
    QString current_unit_name = view_item->axesUnits().getValue();
    return GUI::Util::CoordName::coordFromName(current_unit_name);
}

} // namespace


void GUI::Model::DataViewUtils::updateAxesTitle(Data1DViewItem* view_item)
{
    std::unique_ptr<ICoordSystem> converter{getConverter(view_item)};
    if (!converter)
        return;
    auto current_units = selectedUnits(view_item);
    view_item->setXaxisTitle(QString::fromStdString(converter->axisName(0, current_units)));
    if (converter->dimension() > 1)
        view_item->setYaxisTitle(QString::fromStdString(converter->axisName(1, current_units)));
}

std::unique_ptr<OutputData<double>>
GUI::Model::DataViewUtils::getTranslatedData(Data1DViewItem* view_item, DataItem* data_item)
{
    std::unique_ptr<OutputData<double>> result;
    if (!data_item || !data_item->getOutputData())
        return result;

    auto* converter = getConverter(view_item);
    auto current_units = selectedUnits(view_item);

    result = converter->createOutputData(current_units);
    result->setRawDataVector(data_item->getOutputData()->getRawDataVector());

    return result;
}
